#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'jmeter_script_template.py'
__create_time__ = '2020/12/24 14:44'

from typing import Optional
from re import findall
from urllib.parse import urlparse
from xml.sax.saxutils import escape
from xiaobaiauto2.utils.xiaobaiauto2Installer import getWebList

NEED_EXTRACTOR = False
EXTRACTOR_NAME = []
REQUEST_NAME_LIST = []

def JMETER_SCRIPT_HEAD(JMETER_VERSION: Optional[str] = '5.4.3'):
    return f'''<?xml version="1.0" encoding="UTF-8"?>
<jmeterTestPlan version="1.2" properties="5.0" jmeter="{JMETER_VERSION}">
  <hashTree>
    <TestPlan guiclass="TestPlanGui" testclass="TestPlan" testname="测试计划" enabled="true">
      <stringProp name="TestPlan.comments"></stringProp>
      <boolProp name="TestPlan.functional_mode">false</boolProp>
      <boolProp name="TestPlan.tearDown_on_shutdown">true</boolProp>
      <boolProp name="TestPlan.serialize_threadgroups">false</boolProp>
      <elementProp name="TestPlan.user_defined_variables" elementType="Arguments" guiclass="ArgumentsPanel" testclass="Arguments" testname="用户定义的变量" enabled="true">
        <collectionProp name="Arguments.arguments"/>
      </elementProp>
      <stringProp name="TestPlan.user_define_classpath"></stringProp>
    </TestPlan>
    <hashTree>
      <CookieManager guiclass="CookiePanel" testclass="CookieManager" testname="HTTP Cookie管理器" enabled="true">
        <collectionProp name="CookieManager.cookies"/>
        <boolProp name="CookieManager.clearEachIteration">false</boolProp>
        <boolProp name="CookieManager.controlledByThreadGroup">false</boolProp>
      </CookieManager>
      <hashTree/>
      <ResultCollector guiclass="ViewResultsFullVisualizer" testclass="ResultCollector" testname="察看结果树" enabled="true">
        <boolProp name="ResultCollector.error_logging">false</boolProp>
        <objProp>
          <name>saveConfig</name>
          <value class="SampleSaveConfiguration">
            <time>true</time>
            <latency>true</latency>
            <timestamp>true</timestamp>
            <success>true</success>
            <label>true</label>
            <code>true</code>
            <message>true</message>
            <threadName>true</threadName>
            <dataType>true</dataType>
            <encoding>false</encoding>
            <assertions>true</assertions>
            <subresults>true</subresults>
            <responseData>false</responseData>
            <samplerData>false</samplerData>
            <xml>false</xml>
            <fieldNames>true</fieldNames>
            <responseHeaders>false</responseHeaders>
            <requestHeaders>false</requestHeaders>
            <responseDataOnError>false</responseDataOnError>
            <saveAssertionResultsFailureMessage>true</saveAssertionResultsFailureMessage>
            <assertionsResultsToSave>0</assertionsResultsToSave>
            <bytes>true</bytes>
            <sentBytes>true</sentBytes>
            <url>true</url>
            <threadCounts>true</threadCounts>
            <idleTime>true</idleTime>
            <connectTime>true</connectTime>
          </value>
        </objProp>
        <stringProp name="filename"></stringProp>
      </ResultCollector>
      <hashTree/>
      <ThreadGroup guiclass="ThreadGroupGui" testclass="ThreadGroup" testname="线程组" enabled="true">
        <stringProp name="ThreadGroup.on_sample_error">continue</stringProp>
        <elementProp name="ThreadGroup.main_controller" elementType="LoopController" guiclass="LoopControlPanel" testclass="LoopController" testname="循环控制器" enabled="true">
          <boolProp name="LoopController.continue_forever">false</boolProp>
          <stringProp name="LoopController.loops">1</stringProp>
        </elementProp>
        <stringProp name="ThreadGroup.num_threads">1</stringProp>
        <stringProp name="ThreadGroup.ramp_time">1</stringProp>
        <boolProp name="ThreadGroup.scheduler">false</boolProp>
        <stringProp name="ThreadGroup.duration"></stringProp>
        <stringProp name="ThreadGroup.delay"></stringProp>
        <boolProp name="ThreadGroup.same_user_on_next_iteration">true</boolProp>
      </ThreadGroup>
      <hashTree>'''

def JMETER_SCRIPT_FOOT():
    return '''
      </hashTree>
    </hashTree>
  </hashTree>
</jmeterTestPlan>'''

def JMETER_SCRIPT_HEADER(key, value):
    try:
        key = escape(key)
    except Exception as e:
        pass
    try:
        value = escape(value)
    except Exception as e:
        pass
    return f'''
              <elementProp name="" elementType="Header">
                <stringProp name="Header.name">{key}</stringProp>
                <stringProp name="Header.value">{value}</stringProp>
              </elementProp>'''

def JMETER_SCRIPT_EXTRACTOR(t: Optional[str] = 're',
                            name: Optional[str] = None,
                            match: Optional[str] = None,
                            match_num: Optional[int] = 1):
    if name in [None, ''] or match in [None, '']:
        return ''
    else:
        try:
            name = escape(name.strip())
        except Exception as e:
            pass
        try:
            match = escape(match.strip())
        except Exception as e:
            pass
    if 're' == t:
        return f'''
          <RegexExtractor guiclass="RegexExtractorGui" testclass="RegexExtractor" testname="{name}正则提取器" enabled="true">
            <stringProp name="RegexExtractor.useHeaders">false</stringProp>
            <stringProp name="RegexExtractor.refname">{name}</stringProp>
            <stringProp name="RegexExtractor.regex">{match}</stringProp>
            <stringProp name="RegexExtractor.template">$1$</stringProp>
            <stringProp name="RegexExtractor.default"></stringProp>
            <stringProp name="RegexExtractor.match_number">{match_num}</stringProp>
          </RegexExtractor>
          <hashTree/>'''
    elif 'json' == t:
        return f'''
          <JSONPostProcessor guiclass="JSONPostProcessorGui" testclass="JSONPostProcessor" testname="{name}JSON提取器" enabled="true">
            <stringProp name="JSONPostProcessor.referenceNames">{name}</stringProp>
            <stringProp name="JSONPostProcessor.jsonPathExprs">{match}</stringProp>
            <stringProp name="JSONPostProcessor.match_numbers">{match_num}</stringProp>
          </JSONPostProcessor>
          <hashTree/>'''

def JMETER_EQUEST(file_path: Optional[str] = '',
                  post_value: Optional[str] = '',
                  contentEncoding: Optional[str] = '',
                  url: Optional[str] = '/',
                  method: Optional[str] = 'GET',
                  headers: Optional[dict] = None,
                  response: Optional[str] = ''):
    global NEED_EXTRACTOR, EXTRACTOR_NAME, REQUEST_NAME_LIST
    handle = ''
    header = ''
    if isinstance(headers, dict):
        for k, v in headers.items():
            header += JMETER_SCRIPT_HEADER(k, v)
    HTTPer = urlparse(url=url)
    scheme = HTTPer.scheme
    netloc = HTTPer.netloc.split(':')[0]
    if HTTPer.port is not None:
        port = HTTPer.port
    else:
        if HTTPer.scheme == 'https':
            port = '443'
        elif HTTPer.scheme == 'http':
            port = '80'
        else:
            port = ''
    if HTTPer.query != '':
        if HTTPer.fragment != '':
            path = f'{HTTPer.path}?{HTTPer.query}#{HTTPer.fragment}'
        else:
            path = f'{HTTPer.path}?{HTTPer.query}'
    else:
        path = HTTPer.path
    request_name = 'index' if HTTPer.path in ['', '/'] else HTTPer.path.split('/')[-1]
    if request_name not in REQUEST_NAME_LIST:
        REQUEST_NAME_LIST.append(request_name)
    else:
        _i = 1
        while request_name + '_' + _i.__str__() in REQUEST_NAME_LIST:
            _i += 1
        REQUEST_NAME_LIST.append(request_name + '_' + _i.__str__())
        request_name = request_name + '_' + _i.__str__()
    if NEED_EXTRACTOR and response != '' and len(EXTRACTOR_NAME) > 0:
        '''
            json:
                '{"name": "Tser", "age": 30}'
                '{'name': 'Tser', 'age': 30}'
            xml/html:
                <man><name>Tser</name><age>30</age></man>
                <man name="Tser" age="30"></man>
                <man name='Tser' age='30'></man>
            text:
                name: Tser, age: 30
                name=Tser, age=30
        '''
        EXTRACTORED_NAME = []
        for key in EXTRACTOR_NAME:
            if key in response and NEED_EXTRACTOR:
                m = findall(f'({key}[\"\'=:\s0-9a-zA-Z}}>/]+)', response)
                mat = m[0] if len(m) > 0 else ''
                if '=' in mat:
                    fsp = '='
                elif ':' in mat:
                    fsp = ':'
                else:
                    fsp = ' '
                ma_list = mat.split(fsp)
                if '"' in ma_list[1]:
                    sp = '"'
                elif "'" in ma_list[1]:
                    sp = "'"
                else:
                    sp = ' '
                ma1_list = ma_list[1].split(sp)
                if len(ma1_list) >= 3:
                    match = ''.join([ma_list[0], fsp, ma1_list[0], sp, "(.+?)", sp, ma1_list[2]])
                elif len(ma1_list) == 1:
                    match = ''.join([ma_list[0], fsp, sp, "(.+?)", sp])
                else:
                    val0 = findall(f'"{key}": "(.+?)"', response)
                    val1 = findall(f"'{key}': '(.+?)'", response)
                    val2 = findall(f'{key}="(.+?)"', response)  # HTML/XML属性中
                    val3 = findall(f"{key}='(.+?)'", response)
                    val4 = findall(f'{key}:"(.+?)"', response)
                    val5 = findall(f"{key}:'(.+?)'", response)
                    val6 = findall(f'{key}=([0-9a-zA-Z]+?)', response)  # HTML/XML属性中
                    if len(val0) > 0:
                        match = f'"{key}": "(.+?)"'
                    elif len(val1) > 0:
                        match = f"'{key}': '(.+?)'"
                    elif len(val2) > 0:
                        match = f'{key}="(.+?)"'
                    elif len(val3) > 0:
                        match = f"{key}='(.+?)'"
                    elif len(val4) > 0:
                        match = f'{key}:"(.+?)"'
                    elif len(val5) > 0:
                        match = f"{key}:'(.+?)'"
                    elif len(val6) > 0:
                        match = f'{key}=([0-9a-zA-Z]+?)'
                    else:
                        match = ''
                handle += JMETER_SCRIPT_EXTRACTOR(name=key, match=match)
                if match != '':
                    EXTRACTORED_NAME.append(key)
        for k in EXTRACTORED_NAME:
            EXTRACTOR_NAME.remove(k)
        if EXTRACTOR_NAME == []:
            NEED_EXTRACTOR = False
    if '${' in post_value or '${' in header:
        if NEED_EXTRACTOR is False:
            NEED_EXTRACTOR = True
        EXTRACTOR_NAME.extend(findall('\${(.+?)}', post_value))
        EXTRACTOR_NAME.extend(findall('\${(.+?)}', header))
    return f'''
        <HTTPSamplerProxy guiclass="HttpTestSampleGui" testclass="HTTPSamplerProxy" testname="{escape(request_name)}" enabled="true">
          <elementProp name="HTTPsampler.Files" elementType="HTTPFileArgs">
            <collectionProp name="HTTPFileArgs.files">
              <elementProp name="{escape(file_path)}" elementType="HTTPFileArg">
                <stringProp name="File.path">{escape(file_path)}</stringProp>
                <stringProp name="File.paramname"></stringProp>
                <stringProp name="File.mimetype"></stringProp>
              </elementProp>
            </collectionProp>
          </elementProp>
          <boolProp name="HTTPSampler.postBodyRaw">true</boolProp>
          <elementProp name="HTTPsampler.Arguments" elementType="Arguments">
            <collectionProp name="Arguments.arguments">
              <elementProp name="" elementType="HTTPArgument">
                <boolProp name="HTTPArgument.always_encode">false</boolProp>
                <stringProp name="Argument.value">{escape(post_value)}</stringProp>
                <stringProp name="Argument.metadata">=</stringProp>
              </elementProp>
            </collectionProp>
          </elementProp>
          <stringProp name="HTTPSampler.domain">{escape(netloc)}</stringProp>
          <stringProp name="HTTPSampler.port">{port}</stringProp>
          <stringProp name="HTTPSampler.protocol">{scheme}</stringProp>
          <stringProp name="HTTPSampler.contentEncoding">{contentEncoding}</stringProp>
          <stringProp name="HTTPSampler.path">{escape(path)}</stringProp>
          <stringProp name="HTTPSampler.method">{method}</stringProp>
          <boolProp name="HTTPSampler.follow_redirects">true</boolProp>
          <boolProp name="HTTPSampler.auto_redirects">false</boolProp>
          <boolProp name="HTTPSampler.use_keepalive">true</boolProp>
          <boolProp name="HTTPSampler.DO_MULTIPART_POST">false</boolProp>
          <stringProp name="HTTPSampler.embedded_url_re"></stringProp>
          <stringProp name="HTTPSampler.connect_timeout"></stringProp>
          <stringProp name="HTTPSampler.response_timeout"></stringProp>
        </HTTPSamplerProxy>
        <hashTree>
          <HeaderManager guiclass="HeaderPanel" testclass="HeaderManager" testname="HTTP信息头管理器" enabled="true">
            <collectionProp name="HeaderManager.headers">
              {header}
            </collectionProp>
          </HeaderManager>
          <hashTree/>
          {handle}
        </hashTree>'''

def create_jmeter(request: Optional[list] = None):
    try:
        latest = getWebList(url='https://mirrors.tuna.tsinghua.edu.cn/apache/jmeter/binaries/HEADER.html',
                            match='Apache JMeter (.+?) requires')
        version = latest[0] if len(latest) > 0 else '5.4'
    except Exception as e:
        version = '5.4'
    if request is None:
        return JMETER_SCRIPT_HEAD(version) + JMETER_EQUEST() + JMETER_SCRIPT_FOOT()
    else:
        code = ''
        if len(request) == 1 and isinstance(request[0], dict):
            try:
                code = JMETER_EQUEST(
                    post_value=request[0].get('data'),
                    url=request[0].get('url'),
                    method=request[0].get('method'),
                    headers=request[0].get('headers'),
                    response=request[0].get('response')) + code
            except KeyError as e:
                print(e)
        else:
            for i in range(len(request) - 1, -1, -1):
                v = request[i]
                try:
                    if isinstance(v, dict):
                        code = JMETER_EQUEST(
                            post_value=v.get('data'),
                            url=v.get('url'),
                            method=v.get('method'),
                            headers=v.get('headers'),
                            response=v.get('response')) + code
                except KeyError as e:
                    print(e)
        return JMETER_SCRIPT_HEAD(version) + code + JMETER_SCRIPT_FOOT()


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2Email.py'
__create_time__ = '2020/7/15 23:13'

import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.header import Header
from xiaobaiauto2.config.config import EMAILCONFIG
from datetime import datetime
from os.path import split

def send_email(report=''):
    _emil = EMAILCONFIG()
    if report == '':
        report = _emil.report
    f = open(report, 'rb')
    fcontent = f.read()
    f.close()
    message = MIMEMultipart()
    message['From'] = Header(_emil.sender)
    message['To'] = Header(_emil.receiver[0])
    message['Subject'] = Header(_emil.subject, 'utf-8')
    message.attach(MIMEText(fcontent, 'html', 'utf-8'))
    # 附件1
    _report = MIMEText(fcontent, 'base64', 'utf-8')
    _report["Content-Type"] = 'application/octet-stream'
    _report['Content-Disposition'] = 'attachment; filename="%s"' % split(report)[1]
    message.attach(_report)
    try:
        smtp = smtplib.SMTP()
        smtp.connect(_emil.smtpserver, _emil.smtp_port)
        smtp.login(_emil.username, _emil.password)
        smtp.sendmail(_emil.sender, _emil.receiver, message.as_string())
        print(f"邮件已于：{datetime.now()}发送完成")
        smtp.quit()
    except smtplib.SMTPException as e:
        print(f"邮件已于：{datetime.now()}发送失败", e)


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = '__init__.py'
__create_time__ = '2020/7/15 23:12'

import re
import os
try:
    import winreg
except ModuleNotFoundError as e:
    ''' 非windows系统 '''
import shutil
import xml.dom.minidom
from typing import Union, Optional

step = '\\' if os.name == 'nt' else '/'
grep = 'findstr' if os.name == 'nt' else 'grep'
home = os.environ['HOMEPATH'] if 'HOMEPATH' in os.environ.keys() else os.environ['HOME']
home = 'C:\\' + home if os.name == 'nt' else home

def _get_path(cmd='adb'):
    _path = shutil.which(cmd=cmd)
    if not _path and os.name == 'nt':
        return os.path.dirname(os.path.abspath(__file__)) + step + 'adb' + step + 'adb.exe'
    else:
        return _path

def _get_xpath(x: Union[int, str], y: Union[int, str], deviceName=None):
    x, y = int(x), int(y)
    device = deviceName if deviceName else ''
    ADB_COMMAND = f'"{_get_path("adb")}" '
    os.popen(f'{ADB_COMMAND} {device} shell uiautomator dump /sdcard/ui.xml')
    os.popen(f'{ADB_COMMAND} {device} pull /sdcard/ui.xml "{home}"')
    if os.name == 'nt':
        r = re.findall('003[56]\s+.+max\s+(\d+)', os.popen(cmd=f'{ADB_COMMAND} {device} shell getevent -p').read())
    else:
        r = re.findall('003[56]\s+.+max\s+(\d+)',
                       os.popen(cmd=f'{ADB_COMMAND} {device} shell getevent -p | grep -e "0035" -e "0036" | grep -e "max"').read())
    filePath = f'{home}{step}ui.xml'
    if os.path.isfile(filePath):
        DOM = xml.dom.minidom.parse(filePath)
        coll = DOM.documentElement
        nodes = coll.getElementsByTagName('node')
        path = xpath = ''
        pare_size = int(r[0]) * int(r[1])
        for node in nodes:
            pos = re.findall('(\d+)', node.getAttribute('bounds'))
            if x > int(pos[0]) and y > int(pos[1]) and x < int(pos[2]) and y < int(pos[3]):
                if (int(pos[2]) - int(pos[0])) * (int(pos[3]) - int(pos[1])) > pare_size:
                    continue
                pare_size = (int(pos[2]) - int(pos[0])) * (int(pos[3]) - int(pos[1]))
                className = node.getAttribute('class')
                node_class = className if className not in ["", None] else "*"
                node_text = node.getAttribute('text')
                node_id = node.getAttribute('resource-id')
                node_content = node.getAttribute('content-desc')
                node_index = node.getAttribute('index')
                if node_id != "" and node_text != "":
                    path += f'{node_class}[@resource-id="{node_id}" and @text="{node_text}"]/'
                elif node_id != "" and node_text == "":
                    path += f'{node_class}[@resource-id="{node_id}"]/'
                elif node_id == "" and node_text != "":
                    path += f'{node_class}[@text="{node_text}"]/'
                elif node_content != "":
                    path += f'{node_class}[@content-desc="{node_content}"]/'
                else:
                    path += f'{node_class}[{node_index}]/'
        path_list = path[:-1].split('/')
        for i in range(len(path_list) - 1, 0, -1):
            xpath = f'{path_list[i]}/{xpath}'
            if '[@resource-id="' in path_list[i] or '[@text="' in path_list[i] or '[@content-desc="' in path_list[i]:
                break
        return '//' + xpath[:-1]
    else:
        print('截图文件不存在！！！')
        return None

def getPointImg(x: int, y: int):
    ''' 依据指定的左边进行小范围截图
        x,y
        截图坐标（x-5,y-5,x+5,y+5）
    '''

def findpointImg(target, source):
    ''' 将source在target中进行定位 '''

def del_temp(file=f'{home}{step}ui.xml'):
    if os.path.isfile(file):
        try:
            os.remove(file)
        except PermissionError as e:
            print(e)

def win_reg_WRITE_HKEY_CURRENT_USER(path: Optional[str] = None):
    return winreg.OpenKey(winreg.HKEY_CURRENT_USER, path, 0, winreg.KEY_WRITE)

def win_reg_READ_HKEY_CURRENT_USER(path: Optional[str] = None):
    return winreg.OpenKey(winreg.HKEY_CURRENT_USER, path, 0, winreg.KEY_READ)

def win_reg_READ_HKEY_LOCAL_MACHINE(path: Optional[str] = None):
    return winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, path, 0, winreg.KEY_READ)

def win_reg_WRITE_HKEY_LOCAL_MACHINE(path: Optional[str] = None):
    return winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, path, 0, winreg.KEY_WRITE)

def win_reg_write(path: Optional[str] = None, name: Optional[str] = None,  value: Union[int, str] = None):
    K = win_reg_WRITE_HKEY_CURRENT_USER(path=path)
    try:
        winreg.SetValueEx(K, name, 0, winreg.REG_DWORD, value)
    except Exception as e:
        winreg.SetValueEx(K, name, 0, winreg.REG_SZ, value)
    winreg.CloseKey(K)

def win_reg_read(path: Optional[str] = None, name: Optional[str] = None):
    K = win_reg_READ_HKEY_CURRENT_USER(path=path)
    V = winreg.QueryValueEx(K, name)
    winreg.CloseKey(K)
    return V



import setuptools
from xiaobaiauto2.__version__ import __version__

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="xiaobaiauto2",
    version=__version__,
    author="Tser",
    author_email="807447312@qq.com",
    description="xiaobaiauto2是对自动化框架的第三次更新，" + \
                "功能覆盖UI自动化与API自动化意在帮助对自动化有更多需求且过多时间写代码的人群，" + \
                "让大家的时间花在业务的实现上",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/xiaobaikeji/xiaobaiauto2",
    packages=setuptools.find_packages(),
    keywords="test auto automation xiaobai xiaobaiauto2 test framework",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=[
        "allure-pytest",
        "Appium-Python-Client",
        "jmespath",
        "mitmproxy",
        "pytest",
        "pytest-cov",
        "pytest-sugar",
        "pytest-html",
        "pytest-ordering",
        "pytest-rerunfailures",
        "pytest-xdist",
        "pytest-instafail",
        "pyyaml",
        "requests",
        "selenium",
        "pyperclip"
    ],
    package_data={
        'xiaobaiauto2': [
            'data/favicon.ico',
            'data/xiaobaiauto2.db',
            'test/runTestCase.bat',
            'utils/xiaobaiCaptcha.pyd',
            'utils/adb/adb.exe',
            'utils/adb/AdbWinApi.dll',
            'utils/adb/AdbWinUsbApi.dll'
        ],
    },
    entry_points={'console_scripts': [
        'xiaobaiauto2Timer = xiaobaiauto2.utils.xiaobaiauto2Timer:main',
        'xiaobaiauto2Api = xiaobaiauto2.utils.xiaobaiauto2Tools:cmd',
        'xiaobaiauto2Proxy = xiaobaiauto2.utils.xiaobaiauto2Proxy:cmd',
        'xiaobaiauto2App = xiaobaiauto2.utils.xiaobaiauto2App:adb_cmd',
        'xiaobaiauto2Copy = xiaobaiauto2.utils.xiaobaiauto2Copy:main',
    ]},
)

#python setup.py sdist bdist_wheel

#python -m twine upload dist/*


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test_allure_report_demo.py'
__create_time__ = '2020/12/23 21:32'

import pytest
import allure
import os

@pytest.fixture()
def login():
    print("登录")
    yield
    print("登录完成")

@allure.feature('搜索')
def test_search(login):
    ''' 搜索商品 '''
    print("search")

@allure.feature('支付')
def test_pay():
    ''' 余额支付 '''
    print("pay")

if __name__ =="__main__":
    # 执行测试并生成数据为allure生成报告做准备
    pytest.main(['--alluredir', '.'])
    # 生成测试报告
    os.system('allure serve .')


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'utils.py'
__create_time__ = '2020/11/2 19:15'

from typing import Optional
from yaml import load_all, dump_all
import pysnooper


@pysnooper.snoop()
class ModelBase(object):
    def __init__(self, file: Optional[str] = '', mode: Optional[str] = 'r', encoding: Optional[str] = 'utf-8'):
        '''
            初始化
        :param file:        文件名
        :param model:       模式
        :param encoding:    编码
        '''
        self.obj = open(file=file, mode=mode, encoding=encoding)

    def convert(self, lod: Optional[str], new: Optional[str], encoding: Optional[str] = 'utf-8'):
        '''
            格式转换
        :param lod: json|yaml|yam|xml
        :param new: json|yaml|yam|xml
        :param encoding: utf-8
        :return:
        example:
            convert(old='json', new='yaml')
            or
            convert(old='json', new='yam', encoding='gbk')
        '''

    def json2yaml(self):
        ''' json file convert to yaml file '''
        pass

    def yaml2json(self):
        ''' yaml file convert to json file '''

    def xml2json(self):
        ''' xml file convert to json file '''

    def cmdRun(self):
        ''' pytest run '''
        pass

    def yamlRun(self):
        ''' yaml file run '''
        pass

    def jsonRun(self):
        ''' json file run '''
        pass

class ApiHandle(ModelBase):
    def json2yaml(self):
        pass


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2App.py'
__create_time__ = '2020/12/29 21:41'

import os
from argparse import ArgumentParser
from xiaobaiauto2.utils.xiaobaiauto2Subprocess import syncSub, devices, dataProcess
from xiaobaiauto2.__version__ import __version__
from xiaobaiauto2.utils import grep, _get_path

def adb_cmd():
    ADB_COMMAND = f'"{_get_path("adb")}" '
    arg = ArgumentParser(prog='xiaobaiauto2App [optional] [command]',
                         description=f'小白科技·移动设备调试桥·{__version__}')
    arg.add_argument('-d', '--deviceName', default='', help='多设备场景使用')
    arg.add_argument('-i', '--index', default=0, type=int, help='设备索引，多设备场景使用')
    arg.add_argument('-s', '--save', default='', type=str, help='需要保存的*.py脚本')
    par = arg.parse_args()
    syncSub(ADB_COMMAND + 'devices -l', match='(\S+)\s+device\s+', other='device', filename=par.save)
    if par.deviceName == '' and isinstance(devices, list) and devices.__len__() > 0 or\
        par.deviceName != '' and par.deviceName not in devices:
        deviceName = '-s ' + devices[par.index]
    elif par.deviceName != '' and isinstance(devices, list) and devices.__len__() > 0 and par.deviceName in devices:
        deviceName = '-s ' + par.deviceName
    else:
        deviceName = ''
    version = os.popen(cmd=f'{ADB_COMMAND}{deviceName} shell getprop ro.build.version.release').read()
    syncSub(f'{ADB_COMMAND}{deviceName} shell getprop ro.build.version.release',
            other='version', deviceName=deviceName, filename=par.save)
    if os.name == 'nt':
        syncSub(f'{ADB_COMMAND}{deviceName} shell getevent -p',
                match='003[56]\s+.+max\s+(\d+)', other='size', deviceName=deviceName, filename=par.save)
    else:
        syncSub(f'{ADB_COMMAND}{deviceName} shell getevent -p' + f'| {grep} -e "0035" -e "0036" | {grep} -e "max"',
                match='003[56]\s+.+max\s+(\d+)', other='size', deviceName=deviceName, filename=par.save)
    if [int(v) for v in version.split('.')] >= [int(v) for v in '8.1'.split('.')]:
        syncSub(f'{ADB_COMMAND}{deviceName} shell dumpsys activity | grep ' + '"mResume"',
                match='([\.0-9a-zA-Z]+?/[\.0-9a-zA-Z]+)', other='activity', deviceName=deviceName, filename=par.save)
    else:
        syncSub(f'{ADB_COMMAND}{deviceName} shell dumpsys activity | grep ' + '"mFocus"',
                match='([\.0-9a-zA-Z]+?/[\.0-9a-zA-Z]+)', other='activity', deviceName=deviceName, filename=par.save)
    try:
        syncSub(f'{ADB_COMMAND}{deviceName} shell getevent',
                match='003[56]\s+[0]+([0-9a-f]+)', other='getevent', deviceName=deviceName, filename=par.save)
    except KeyboardInterrupt as e:
        print('您已经手动终止进程.')


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = '__version__.py'
__create_time__ = '2020/7/15 23:21'

VERSION = (0, 3)

__version__ = '.'.join(map(str, VERSION))


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'myMockServer.py'
__create_time__ = '2020/12/18 23:42'

from xiaobaiauto2.xiaobaiauto2 import mock, MockServer

@mock(uri='/login', response={"errcode": 200, "msg": "登录成功", "data": {"token": "123456"}})
@mock(uri='/register', response={"errcode": 200, "msg": "注册成功", "data": {}})
@MockServer(uri='/index', response={"errcode": 200, "msg": "首页成功", "data": {}})
def run(): pass

if __name__ == '__main__':
    run(port=7777)



#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'conftest.py'
__create_time__ = '2020/7/6 14:11'
'''  UI自动化可以使用该文件，其他测试可以移除防止对测试有干扰   '''

from appium import webdriver
from socket import socket
from subprocess import PIPE, Popen
from xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS
import pytest

b = None

@pytest.mark.hookwrapper
def pytest_runtest_makereport(item):
    """
    当测试失败的时候，自动截图，展示到html报告中
    :param item:
    """
    pytest_html = item.config.pluginmanager.getplugin('html')
    outcome = yield
    report = outcome.get_result()
    extra = getattr(report, 'extra', [])
    if report.when == 'call' or report.when == "setup":
        xfail = hasattr(report, 'wasxfail')
        if (report.skipped and xfail) or (report.failed and not xfail):
            file_name = report.nodeid.replace("::", "_")+".png"
            screen_img = _capture_screenshot()
            if file_name:
                html = '<div><img src="data:image/png;base64,%s" alt="screenshot" style="width:600px;height:300px;" ' \
                       'onclick="window.open(this.src)" align="right"/></div>' % screen_img
                extra.append(pytest_html.extras.html(html))
        report.extra = extra

def _capture_screenshot():
    '''
    截图保存为base64，展示到html中
    :return:
    '''
    if b:
        return b.get_screenshot_as_base64()
    else:
        return '未截图'

def _check_port(ip='127.0.0.1', port=4723):
    sock = socket()
    try:
        sock.connect((ip, int(port)))
        sock.shutdown(2)
        return True
    except Exception as e:
        c = Popen(f'appium -a {ip} -p {port}', shell=True, stdout=PIPE)
        print(c.stdout.read())
        return True
    except ChildProcessError as e:
        return False

@pytest.fixture(scope='session', autouse=True)
def mobile():
    global m
    '''
        'automationName': 'uiautomator2',
        'platformName': 'Android',
        'platformVersion': '5.1',  
        'deviceName': '',  
        'noReset': True,  
        'allowClearUserData' = 'true',      #用户可自行选择清除数据
        'fullReset' = "false",              #卸载程序，默认为false
        'exported'="true",                  #是否支持其他应用调用当前组件
        'appPackage': '',  
        'appActivity': '',  
        'unicodeKeyboard': True,            #使用unicode编码方式发送字符串  
        'resetKeyboard': True               #将键盘隐藏起来，输入中文就要增加这两个参数
        'udid':'设备UDID'
        'bundleId':'应用包名'
    '''
    PUBLIC_VARS['android_caps'] = {
        'automationName': 'uiautomator2',
        'platformName': 'Android',
        'platformVersion': '5.1',
        'deviceName': '设备名',
        'noReset': True,
        'allowClearUserData': 'true',
        'fullReset': "false",
        'exported': "true",
        'appPackage': '应用包名',
        'appActivity': '应用Activity名',
        'unicodeKeyboard': True,
        'resetKeyboard': True
    }
    PUBLIC_VARS['ios_caps'] = {
        'platformName': 'iOS',
        'platformVersion': '11.4',
        'deviceName': '设备名',
        'udid': '设备UDID',
        'bundleId': '应用包名',
        'noReset': True,
    }
    if _check_port():
        if m is None:
            m = webdriver.Remote('http://127.0.0.1:4723/wd/hub', PUBLIC_VARS['android_caps'])
        yield m
        m.quit()
    else:
        print('appium服务启动失败，请手动启动服务.')
        exit(0)


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2Timer.py'
__create_time__ = '2020/7/20 21:13'

from os import chdir, path
from subprocess import Popen, PIPE
from threading import Thread
from tkinter import Tk, Label, Entry, Button, Frame, Listbox, Scrollbar, filedialog, END, LEFT, RIGHT, Y, StringVar
from tkinter.messagebox import showerror
from xiaobaiauto2.config.config import TIMERCONFIG
from datetime import datetime
from time import sleep

_timer = TIMERCONFIG()

IS_RUN = False

root = Tk()
root.geometry(_timer.geometry)
root.title(_timer.timerTitle)
root.iconbitmap(_timer.timerpath)

_t = StringVar()
_t.set('* * * * *')
_p = StringVar()
_p.set(path.abspath(path.curdir))
_c = StringVar()
_c.set('pytest --html=report.html --self-contained-html')

def _jiexi(_type='min', _timer='* * * * *'):
    msg = '''
        范围：
        min     0 ... 59
        hour    0 ... 23
        day     1 ... 31
        mou     1 ... 12
        week    1 ... 7
        表达式：
        * or 1,2,3-7/2 or 1-7/2 or 1,2,3-7 or */2 or 1-7 or 1,2,3 or 1
    '''
    _min = list(range(60))
    _hour = list(range(24))
    _day = list(range(1, 32))
    _mou = list(range(1, 13))
    _week = list(range(1, 8))
    _data = {
        'min': _min,
        'hour': _hour,
        'day': _day,
        'mou': _mou,
        'week': _week
    }
    if '*' == _timer:
        return _data[_type]
    elif '-' in _timer and '/' in _timer and ',' in _timer:
        '''1,2,3-7/2'''
        _tmp = []
        for t in _timer.split(','):
            if '-' not in t and '/' not in t:
                _tmp.append(t)
            else:
                _tmp += list(range(int(t.split('/')[0].split('-')[0]),
                                   int(t.split('/')[0].split('-')[1]) + 1,
                                   int(t.split('/')[1])))
        _tmp = list(map(lambda x: int(x), _tmp))
        if set(_tmp).issubset(set(_data[_type])):
            return _tmp
        else:
            showerror(title='错误', message=msg)
            return []
    elif '-' in _timer and '/' in _timer and ',' not in _timer:
        '''1-7/2'''
        _tmp = list(range(int(_timer.split('/')[0].split('-')[0]),
                          int(_timer.split('/')[0].split('-')[1]) + 1,
                          int(_timer.split('/')[1])))
        _tmp = list(map(lambda x: int(x), _tmp))
        if set(_tmp).issubset(set(_data[_type])):
            return _tmp
        else:
            showerror(title='错误', message=msg)
            return []
    elif ',' in _timer and '-' in _timer and '/' not in _timer:
        '''1,2,3-7'''
        _tmp = []
        for t in _timer.split(','):
            if '-' not in t:
                _tmp.append(t)
            else:
                _tmp += list(range(int(t.split('-')[0]),
                                   int(t.split('-')[1]) + 1))
        _tmp = list(map(lambda x: int(x), _tmp))
        if set(_tmp).issubset(set(_data[_type])):
            return _tmp
        else:
            showerror(title='错误', message=msg)
            return []
    elif ',' not in _timer and '-' not in _timer and '/' in _timer:
        '''*/2'''
        _tmp = []
        if '*' == _timer.split('/')[0]:
            return _data[_type][::int(_timer.split('/')[0])]
        else:
            showerror(title='错误', message=msg)
            return []
    elif '-' in _timer and ',' not in _timer and '/' not in _timer:
        '''1-7'''
        _tmp = list(range(int(_timer.split('-')[0]),
                           int(_timer.split('-')[1]) + 1))
        _tmp = list(map(lambda x: int(x), _tmp))
        if set(_tmp).issubset(set(_data[_type])):
            return _tmp
        else:
            showerror(title='错误', message=msg)
            return []
    elif ',' in _timer and '/' not in _timer and '-' not in _timer:
        '''1,2,3'''
        _tmp = list(map(lambda x: int(x), _timer.split(',')))
        if set(_tmp).issubset(set(_data[_type])):
            return _tmp
        else:
            showerror(title='错误', message=msg)
            return []
    elif ',' not in _timer and '/' not in _timer and '-' not in _timer:
        '''1'''
        _tmp = [int(_timer)]
        if set(_tmp).issubset(set(_data[_type])):
            return _tmp
        else:
            showerror(title='错误', message=msg)
            return []
    else:
        showerror(title='错误', message='信息有错误[时间格式]或[命令为空]！')
        return []

def _Job():
    global IS_RUN
    ter = _t.get()
    if ter.split(' ').__len__() == 5:
        '''解析正确的格式的时间数据'''
        if ter.split(' ')[2] == '*' and ter.split(' ')[3] == '*':
            _min_list = _jiexi('min', ter.split(' ')[0])
            _hour_list = _jiexi('hour', ter.split(' ')[1])
            _day_list = _jiexi('day', ter.split(' ')[2])
            _mou_list = _jiexi('mou', ter.split(' ')[3])
            _week_list = _jiexi('week', ter.split(' ')[4])
            IS_RUN = True
            while 1:
                _now = datetime.now()
                _now_min = _now.minute
                _now_hour = _now.hour
                _now_day = _now.day
                _now_mou = _now.month
                _now_week = _now.weekday() + 1
                if _now_week in _week_list:
                    if _now_hour in _hour_list:
                        if _now_min in _min_list:
                            cmd = _c.get()
                            cur_path = _p.get()
                            if cur_path != '':
                                chdir(cur_path)
                            else:
                                chdir(path.abspath(path.curdir))
                            if '' != cmd:
                                lb.delete(0, END)
                                try:
                                    lb.insert(END, '...命令执行中...')
                                    try:
                                        pcmd = Popen(cmd, shell=True, stdout=PIPE,
                                                     stderr=PIPE)
                                        lb.delete(0, END)
                                        for i in iter(pcmd.stdout.readline, 'b'):
                                            if not i:
                                                break
                                            lb.insert(END, i.decode('gbk'))
                                    except Exception as e:
                                        print('路径有问题')
                                except Exception as e:
                                    lb.delete(0, END)
                                    lb.insert(END, e)
                            else:
                                showerror(title='错误', message='信息有错误[时间格式]或[命令为空]！')
                            sleep(1)
                            # 防止重复执行
                            if _now_min == _now.minute and _now_hour == _now.hour:
                                sleep(60 - _now.second)
                        else:
                            sleep(60)
                    else:
                        sleep(60 * 60)
                else:
                    sleep(24 * 60 * 60)
        else:
            showerror(title='错误', message='信息有错误[时间格式暂不支持日、月]！')
    else:
        showerror(title='错误', message='信息有错误[时间格式]或[命令为空]！')

def _check_title():
    while 1:
        if IS_RUN:
            root.title(_timer.timerRuningTitle + str(datetime.now())[:-7])
        else:
            root.title(_timer.timerTitle + str(datetime.now())[:-7])
        sleep(1)

def _choose_script_dir():
    global _p
    dir_name = filedialog.askdirectory()
    if '' != dir_name:
        _p.set(dir_name)

def _run():
    t1 = Thread(target=_Job,)
    t1.setDaemon(True)
    t1.start()

frame0 = Frame(master=root)
frame00 = Frame(master=frame0, width=75)
frame01 = Frame(master=frame0, width=75)
frame2 = Frame(master=root, width=75, height=50)
Label(master=frame00, text='定时表达式：').pack()
Entry(master=frame00, textvariable=_t, width=20).pack()
Button(master=frame01, text='脚本路径', command=_choose_script_dir).pack()
Entry(master=frame01, textvariable=_p, width=45).pack()
frame0.pack()
frame00.pack(side=LEFT, fill=Y)
frame01.pack(side=RIGHT, fill=Y)
Label(master=root, text='执行命令：').pack()
Entry(master=root, textvariable=_c, width=65).pack()
Button(master=root, text='开始运行', command=_run).pack()
lb = Listbox(frame2, width=65, height=50)
scr = Scrollbar(frame2)
lb.config(yscrollcommand=scr.set)
scr.config(command=lb.yview)
lb.pack(side=LEFT, fill=Y)
lb.insert(END, '...运行日志...')
scr.pack(side=RIGHT, fill=Y)
frame2.pack()

def main():
    t0 = Thread(target=_check_title, )
    t0.setDaemon(True)
    t0.start()
    root.mainloop()


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test02.py'
__create_time__ = '2020/7/15 23:34'

import pytest

'''通过order=序号，first与last关键词将脚本进行顺序运行'''

@pytest.mark.run(order=2)
def test_Case1():
    '''操作步骤'''

@pytest.mark.run(order=3)
def test_Case2():
    '''操作步骤'''

@pytest.mark.last
def test_last():
    '''测试结束了，发个邮件吧'''

@pytest.mark.run(order=1)
def test_first():
    '''测试开始了，准备邮件信息'''